; Show :
;	* gray mode
;	* Sprite mask routine 16
;	* Joypad
;	* Cls
;	* Perfect Synchro
;	* Triangle
;	* double Buffering
;	* Internal Handle Stack

; Sample test for genlib.
; Just print a ship that you can move.

	xdef	_main
	xdef	_comment
	xdef	_ti92plus
	xdef	_ti89
	
	include "tios.h"
	include "genlib.h"

; Expand point structure.
xmin	EQU	4
xmax	EQU	6
xdir	EQU	8
ymin	EQU	10
ymax	EQU	12
ydir	EQU	14
pt_s	EQU	16

_main:	jsr	genlib::init			; Init

	genlib::PUSH_DSCREEN	d0		; Allocate a DScreen (Two screens)
	jsr	genlib::set_dscreen_int		; The auto-int will work with the DScreen allocated
	jsr	genlib::set_dscreen_function	; The functions of genlib will work withe this DScreen
	move.l	d0,Screen
	jsr	genlib::init_dscreen		; Alloc a new DScreen for double buffer
	move.l	a0,Screen2			; and the adress
	tst.w	d0
	beq	memory_error			; Memory ?
	jsr	genlib::push_hd			; in the internal handle list (max = 20 handles!)

	; Set Sprite Layer
	clr.w	genlib::sprite_scr_x		; Sprite_plane_x = 0
	clr.w	genlib::sprite_scr_y		; Sprite_plane_y = 0
	move.l	#Tile,genlib::sprite_tile_adr

\loop:
	jsr	genlib::read_joypad		; Read the joypad (Output in d0.l)
	; Test the keys and move the ship
	btst.l	#exit_key,d0
	beq	quit
	btst.l	#up_key,d0
	bne.s	\no_up
		subq.w	#1,Y
\no_up:	btst.l	#down_key,d0
	bne.s	\no_down
		addq.w	#1,Y
\no_down:
	btst.l	#left_key,d0
	bne.s	\no_left
		subq.w	#1,X
\no_left:
	btst.l	#right_key,d0
	bne.s	\no_right
		addq.w	#1,X
\no_right:

	; Update the position of the points
	lea	P1(Pc),a0
	moveq	#4,d1
\lp2		bsr	Point			; Update the position
		lea	pt_s(a0),a0
		dbf	d1,\lp2

	; Start Graphics rendering
\synchro1
		tst.l	genlib::timer		; Wait for the interrupt...
		beq.s	\synchro1
			
	jsr	genlib::cls			; Clear the DScreen

	; Put a triangle
	lea	P1(Pc),a1	;  Point 1
	lea	P2(Pc),a2	;  Point 2
	lea	P3(Pc),a3	;  Point 3
	moveq	#1,d0		; Color = 1 = LightGray
	jsr	genlib::draw_c_face	; Render the face
	lea	P4(Pc),a3	; New Point 3
	moveq	#2,d0		; New color = DarkGray
	jsr	genlib::draw_c_face	; Render the face
	lea	P5(Pc),a2	; New Point 2
	moveq	#3,d0		; New color = Black
	jsr	genlib::draw_c_face	; Render the face

	; Put the ship
	move.w	X(pc),d0			; X coordinate
	move.w	Y(pc),d1			; Y coordinate
	moveq	#7,d2				; Sprite 1
	jsr	genlib@006B		; Put the ship

	; Wait next frame
\wait:
		tst.w	genlib::frame_timer
		beq.s	\wait
	clr.l	genlib::timer
	clr.w	genlib::frame_timer

	; Double_bufferring
	move.w	temps(pc),d1		; Read temps
	move.l	Screen(pc,d1.w),d0	; Read a screen
	jsr	genlib::set_dscreen_int	; Set the interrupt to this Dscreen
	eor.w	#4,d1			; Next screen
	move.l	Screen(PC,d1.w),d0	; Read the other screen
	jsr	genlib::set_dscreen_function	; Set the functions to this DScreen
	move.w	d1,temps		; Save temps
	bra	\loop			; Return to loop

Screen	dc.l	0		; Screen 1
Screen2	dc.l	0		; Screen 2
temps	dc.w	0		; Temps

quit:
memory_error:
	jsr	genlib::free_hd	; Free the DScreen handle
	genlib::POP_DSCREEN	; Desallocate the DScreen
	jsr	genlib::quit	; Quit genlib
	rts			; Quit the program

; In:
;	a0 -> Point
; Out:
;	a0 -> Updated point
; Destroy:
;	d0
Point:
	; Update the Y coordinate
	move.w	ye(a0),d0	; Read Y
	add.w	ydir(a0),d0	; add the direction
	cmp.w	ymin(a0),d0	; Minimum ?
	ble.s	\pasok1
		cmp.w	ymax(a0),d0	; Maximum ?
		blt.s	\ok2	
\pasok1:
	neg.w	ydir(a0)	; Exchange the dir
	bra.s	\done

\ok2:	move.w	d0,ye(a0)	; Save the Y coordinate
\done:
	; Update the x coordinate
x:
	move.w	xe(a0),d0
	add.w	xdir(a0),d0
	cmp.w	xmin(a0),d0
	ble.s	\pasok1
	cmp.w	xmax(a0),d0
	blt.s	\ok2
\pasok1:
	neg.w	xdir(a0)
	bra.s	\done
\ok2:	move.w	d0,xe(a0)
\done:
	rts		

Tile:	incbin "decor.bin"

X	dc.w	10
Y	dc.w	0

P1	dc.w	120,64
	dc.w	-5,250,1
	dc.w	-5,140,2
P2	dc.w	80,50
	dc.w	-50,230,2
	dc.w	0,100,2
P3	dc.w	70,80
	dc.w	5,240,1
	dc.w	-50,100,2
P4	dc.w	150,55
	dc.w	-15,250,-1
	dc.w	-15,140,2
P5	dc.w	140,85
	dc.w	-50,230,-2
	dc.w	-100,50,2

_comment:
	dc.b "GenLib sample exemple",0

	end